Index: firmware/src/os/wdt.c
===================================================================
--- firmware/src/os/wdt.c	(revision 66923)
+++ firmware/src/os/wdt.c	(revision 66924)
@@ -34,13 +34,14 @@
 
 static void wdt_irq(u_int32_t sr)
 {
-  int loopX;
+  int loopX, loopY;
   if (sr & 1)
     AT91F_DBGU_Frame("================> WATCHDOG EXPIRED !!!!!\n\r");
   if (sr & 2)
     AT91F_DBGU_Frame("================> WATCHDOG ERROR !!!!!\n\r");
   
-  while (1) {
+  loopY = 0;
+  while (loopY++ < 30) {
     led_switch(1, 0);
     led_switch(2, 1);
     loopX = 0;
@@ -54,10 +55,59 @@
       asm(" nop ");
 	  
   }
+
+  // Reset
+  AT91F_RSTSetMode(AT91C_BASE_RSTC, (0xF << 8) | // ERSTL ~ 2 seconds
+		   (1 << 16)); // Brown out Interrupt enable
+  AT91F_RSTSoftReset(AT91C_BASE_RSTC, 0xF); // Reset peripheral, processor and External
 }
 
+static void rstc_irq(u_int32_t sr) {
+  int loopX, loopY;
+  
+  loopY = 0;
+  while (loopY++ < 5) {
+    led_switch(1, 1);
+    led_switch(2, 1);
+    loopX = 0;
+    while (loopX++ < 0x60000)
+      asm(" nop ");
+	  
+    led_switch(1, 0);
+    led_switch(2, 0);
+    loopX = 0;
+    while (loopX++ < 0x60000)
+      asm(" nop ");
+	  
+  }
+
+  // Reset
+  AT91F_RSTSetMode(AT91C_BASE_RSTC, (0xF << 8) | // ERSTL ~ 2 seconds
+		   (1 << 16)); // Brown out Interrupt enable
+  AT91F_RSTSoftReset(AT91C_BASE_RSTC, 0xF); // Reset peripheral, processor and External
+}
+
+#define dis_x_WDT_SIMULATE_TIMEOUT
+
+#ifdef WDT_SIMULATE_TIMEOUT
+static unsigned wdt_restart_counter = 0;
+#define WDT_COUNTER_MAX  100000
+#endif
+
 void wdt_restart(void)
 {
+#ifdef WDT_SIMULATE_TIMEOUT
+  if (wdt_restart_counter < WDT_COUNTER_MAX) {
+    if (wdt_restart_counter % 10000 == 0)
+      DEBUGPCR("WatchDog counter = %u", wdt_restart_counter);
+    else if (wdt_restart_counter == WDT_COUNTER_MAX - 1)
+      DEBUGPCR("Final WatchDog counter = %u", wdt_restart_counter);
+    wdt_restart_counter++;
+  } else {
+    return;
+  }
+#endif
+
 	AT91F_WDTRestart(AT91C_BASE_WDTC);
 }
 
@@ -64,6 +114,7 @@
 void wdt_init(void)
 {
 	sysirq_register(AT91SAM7_SYSIRQ_WDT, &wdt_irq);
+	sysirq_register(AT91SAM7_SYSIRQ_RSTC, &rstc_irq);
 
 #ifdef WDT_DEBUG
 	AT91F_WDTSetMode(AT91C_BASE_WDTC, (WDT_WDD << 16) |
